package org.global;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import org.apache.commons.io.FileUtils;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellType;
import org.apache.poi.ss.usermodel.DateUtil;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.manager.PageManager;
import org.openqa.selenium.JavascriptExecutor;
import org.openqa.selenium.OutputType;
import org.openqa.selenium.TakesScreenshot;
import org.openqa.selenium.WebDriver;
import org.openqa.selenium.WebElement;
import org.openqa.selenium.chrome.ChromeDriver;
import org.openqa.selenium.edge.EdgeDriver;
import org.openqa.selenium.interactions.Actions;
import org.openqa.selenium.support.ui.Select;
import org.testng.annotations.AfterTest;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.BeforeTest;
import org.testng.annotations.Optional;
import org.testng.annotations.Parameters;

import com.aventstack.extentreports.ExtentReports;
import com.aventstack.extentreports.ExtentTest;
import com.aventstack.extentreports.model.Media;
import com.aventstack.extentreports.reporter.ExtentSparkReporter;

import io.github.bonigarcia.wdm.WebDriverManager;

public class BaseClass {

	public static BaseClass base = new BaseClass();
	public static PageManager pageManager = new PageManager();
	
	public static ExtentSparkReporter spark = new ExtentSparkReporter("target\\extentTestReport.html");
	public static ExtentReports extend = new ExtentReports();
	public static ExtentTest test;

	@AfterTest
	private void afterTest() {
		extend.flush();
	}

	@BeforeTest
	private void beforeTest() {
		extend.attachReporter(spark);
		test = extend.createTest("AdactinLoginPageValidation");
		test.assignAuthor("Jack");
		test.assignCategory("SmokeTest");
		test.assignDevice("Chrome");

	}

	
	public String screenCapture() throws IOException {

		TakesScreenshot takesScreenshot = (TakesScreenshot) driver;
		File sourceFile = takesScreenshot.getScreenshotAs(OutputType.FILE);
		File targetFile = new File("ErrorImage\\" + System.currentTimeMillis() + ".png");
		FileUtils.copyFile(sourceFile, targetFile);
		String absolutePath = targetFile.getAbsolutePath();
		return absolutePath;

	}
	
	

	@Parameters({ "browserType", "url" })
	@BeforeClass
	public static void browserLaunch_UrlLaunch(@Optional("Chrome") String browser,
			@Optional("https://adactinhotelapp.com/") String browserUrl) {

		try {

			base.getDriver(browser);
			base.getUrl(browserUrl);

			base.excelReport1(2, 3, "LoginCrdentials are Correct");

			Thread.sleep(5000);
		} catch (Exception e) {

		}

		// To Validate User is in Login Page or not
		try {
			if (pageManager.getLoginPage().getUserLogin().isDisplayed()) {

				System.out.println("User Is In Login Page");

				if (pageManager.getLoginPage().getUserLogin().getText().contains("Build 1")) {

					System.out.println("User Is Is Build 1 Login Page");
				}

			} else {

				System.out.println("User Not In Login Page");

			}

		} catch (Exception e) {

		}

	}

	
	public static WebDriver driver;
	public static Actions actions;
	public static JavascriptExecutor javascriptExecutor;
	public static File file;
	public static Workbook book;
	public static Sheet sheet;
	public static Row row;
	public static Cell cell;

	public void getDriver(String browserType) {

		switch (browserType) {
		case "Chrome":

			WebDriverManager.chromedriver().setup();
			driver = new ChromeDriver();
			break;

		case "Edge":

			WebDriverManager.edgedriver().setup();
			driver = new EdgeDriver();
			break;

		default:
			System.out.println("InValid_BrowserType");
			break;
		}

	}

	public void getUrl(String url) {

		driver.get(url);
	}

	public void windowMax() {
		driver.manage().window().maximize();
	}

	public void navigation(String commands) {

		switch (commands) {
		case "refresh":
			driver.navigate().refresh();

			break;

		case "forward":
			driver.navigate().forward();

			break;

		case "back":
			driver.navigate().back();

			break;

		default:
			System.out.println("In Valid Commands");
			break;
		}

	}

	public void sendKeysByJava(WebElement element, String keysToSend) {
		element.sendKeys(keysToSend);
	}

	public void sendKeysByJS(WebElement element, String keysToSend) {
		javascriptExecutor = (JavascriptExecutor) driver;
		javascriptExecutor.executeScript("arguments[0].setAttribute('value','" + keysToSend + "')", element);

	}

	public void clickByJS(WebElement element) {
		javascriptExecutor = (JavascriptExecutor) driver;
		javascriptExecutor.executeScript("arguments[0].click()", element);

	}

	public void scrollByJS(String scrollType, WebElement element) {

		javascriptExecutor = (JavascriptExecutor) driver;

		switch (scrollType) {
		case "Down":

			javascriptExecutor.executeScript("arguments[0].scrollIntoView(true)", element);

			break;

		case "Up":
			javascriptExecutor.executeScript("arguments[0].scrollIntoView(false)", element);

			break;

		default:
			break;
		}

	}

	public void selectByValue(WebElement element, String value) {
		Select select = new Select(element);
		select.selectByValue(value);
	}

	public void actionsMouseOver(WebElement element) {
		actions = new Actions(driver);
		actions.moveToElement(element).build().perform();

	}

	public void actionsByDragandDrop(WebElement source, WebElement target) {
		actions = new Actions(driver);
		actions.dragAndDrop(source, target).build().perform();
	}



	public void windowsHandling(int windowsIndex) {

		String parentWindowsID = driver.getWindowHandle();
		Set<String> allWindowsID = driver.getWindowHandles();

		// To Create New List & Add all the values from Set to List
		List<String> list = new LinkedList<String>();
		list.addAll(allWindowsID);

		// To get Windows ID
		String childWindowsID = list.get(windowsIndex);
		// To Switch to Windows based on requirement
		driver.switchTo().window(childWindowsID);

	}

	public String getWindowsID(int windowsIndex) {

		String parentWindowsID = driver.getWindowHandle();
		Set<String> allWindowsID = driver.getWindowHandles();

		// To Create New List & Add all the values from Set to List
		List<String> list = new LinkedList<String>();
		list.addAll(allWindowsID);

		// To get Windows ID
		String childWindowsID = list.get(windowsIndex);

		return childWindowsID;
	}

	public void switchWindows(String windowsID) {

		driver.switchTo().window(windowsID);
	}

	public void switchFrame(WebElement element) {
		driver.switchTo().frame(element);
	}

	public String getText(WebElement element) {
		String text = element.getText();
		return text;
	}

	public String getAttributeByJava(WebElement element) {
		String attribute = element.getAttribute("value");
		return attribute;
	}

	public Object getAttributeByJS(WebElement element) {
		javascriptExecutor = (JavascriptExecutor) driver;
		Object executeScript = javascriptExecutor.executeScript("return arguments[0].getAttribute('value')", element);
		return executeScript;
	}

	public String readExcel(int rownum, int cellnum) throws IOException {

		// To Locate File
		file = new File("DataBase\\LoginDatas.xlsx");

		// To get File as a Iinput Data
		FileInputStream fileInputStream = new FileInputStream(file);// throws FileNotFoundException

		// To Define Format of Workbook
		book = new XSSFWorkbook(fileInputStream); // throws IOException

		// To get sheet from Book
		sheet = book.getSheet("AdactinCredentials");

		// To get Row from Sheet
		row = sheet.getRow(rownum);

		// To get Cell From Row
		cell = row.getCell(cellnum);

		// To Define Cell Type
		CellType cellType = cell.getCellType();

		String value = null;

		switch (cellType) {
		case STRING:
			value = cell.getStringCellValue();

			break;

		case NUMERIC:

			if (DateUtil.isCellDateFormatted(cell)) {

				Date dateCellValue = cell.getDateCellValue();
				SimpleDateFormat simpleDateFormat = new SimpleDateFormat("dd/MM/yyyy");
				value = simpleDateFormat.format(dateCellValue);

			} else {

				double numericCellValue = cell.getNumericCellValue();
				BigDecimal valueOf = BigDecimal.valueOf(numericCellValue);
				value = valueOf.toString();

			}

			break;

		default:
			System.out.println("In Valid Cell Type");

			break;
		}

		return value;
	}

	public void excelReport(WebElement element, int rownum, int column) throws IOException {

		FileOutputStream fileOutputStream = new FileOutputStream(file);

		cell = sheet.getRow(rownum).createCell(column);

		if (element.isDisplayed()) {

			cell.setCellValue("PASS");

		} else {
			cell.setCellValue("FAIL");
		}

		book.write(fileOutputStream);

	}

	public void excelReport1(int rownum, int column, String message) throws IOException {

		FileOutputStream fileOutputStream = new FileOutputStream(file);

		cell = sheet.getRow(rownum).createCell(column);

		cell.setCellValue(message);

		book.write(fileOutputStream);

	}

}
